/*
 * test.c
 *
 *  Created on: Sep 27, 2010
 *      Author: christian
 */

#include "dsm-api.h"
#include "dsm-api-const.h"
#include <digitalSTROM/ds.h>
#include <unistd.h>

void BusChange_callback(void *arg, dsid_t *id, int flag)
{
    int i;
    for (i = 0; i < 12; ++i)
    {
        printf("%02x ", id->id[i]);
    }
    if (flag)
    {
        printf("left bus\n");
    }
    else
    {
        printf("joined bus\n");
    }
}

void BusState_callback(void *arg, bus_state_t state)
{
    switch (state)
    {
    case DS485_STATE_ISOLATED:
        printf("STATE: ISOLATED\n");
        break;
    case DS485_STATE_CONNECTED:
        printf("STATE: CONNECTED\n");
        break;
    case DS485_STATE_ACTIVE:
        printf("STATE: ACTIVE\n");
        break;
    case DS485_STATE_JOIN:
        printf("STATE: JOIN\n");
        break;
    default:
        printf("STATE: *UNKNOWN*\n");
        break;
    }
}

void EventDeviceAccessibility_on(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t DeviceId, uint16_t ZoneId, uint16_t VendorId, uint32_t SerialNumber)
{
    int i;
    printf("Device 0x%08x (DeviceId: %d in Zone: %d) became active on dSM", SerialNumber, DeviceId, ZoneId);
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf("\n");
}

void EventDeviceAccessibility_off(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t DeviceId, uint16_t ZoneId, uint16_t VendorId, uint32_t SerialNumber)
{
    int i;
    printf("Device 0x%08x (DeviceId: %d in Zone: %d) became inactive on dSM", SerialNumber, DeviceId, ZoneId);
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf("\n");
}

void ZoneCount_callback(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint8_t ZoneCount)
{
    int i;
    printf("dSM:");
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf(" has ZoneCount: %d (result: %d)\n", ZoneCount, error_code);
}

void EventDeviceStatistics(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t DeviceIndex, uint8_t Offset, uint16_t Value)
{
    printf("Device: %d, Offset %d: 0x%02x\n", DeviceIndex, Offset, Value);
}

void ZoneGroupActionRequest_action_call_scene_request(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t ZoneId, uint8_t GroupId, uint8_t SceneId)
{
    int i;
    printf("Call scene %d in zone %d and group %d from dSM: ", SceneId, ZoneId, GroupId);
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf("\n");
}


int main(int argc, char* argv[])
{
    int result;
    const char connection[] = "tcp://192.168.2.124:8442";
    DsmApiCallback_t callback;
    dsid_t dsid = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };
    dsid_t broadcast;
    SetBroadcastId(broadcast);

    DsmApiHandle_t handle = DsmApiInitialize();

    result = DsmApiOpen(handle, connection, 0);

    BusState_callback_t busstate = &BusState_callback;
    callback.function = (void*)busstate;
    callback.arg = NULL;
    DsmApiSetBusStateCallback(handle, &callback);

    BusChange_callback_t buschange = &BusChange_callback;
    callback.function = (void*)buschange;
    callback.arg = NULL;
    DsmApiSetBusChangeCallback(handle, &callback);

    EventDeviceAccessibility_on_event_callback_t event_device_acc_on = &EventDeviceAccessibility_on;
    callback.function = (void*)event_device_acc_on;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_ACCESSIBILITY, EVENT_DEVICE_ACCESSIBILITY_ON, &callback);

    EventDeviceAccessibility_off_event_callback_t event_device_acc_off = &EventDeviceAccessibility_off;
    callback.function = (void*)event_device_acc_off;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_ACCESSIBILITY, EVENT_DEVICE_ACCESSIBILITY_OFF, &callback);

    ZoneCount_response_callback_t zone_count_callback = &ZoneCount_callback;
    callback.function = (void*)zone_count_callback;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_RESPONSE, ZONE_COUNT, 0, &callback);

    EventDeviceStatistics_event_callback_t event_statistics_callback = &EventDeviceStatistics;
    callback.function = (void*)event_statistics_callback;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_STATISTICS, 0, &callback);

    ZoneGroupActionRequest_action_call_scene_request_callback_t scene_callback = &ZoneGroupActionRequest_action_call_scene_request;
    callback.function = (void*)scene_callback;
    callback.arg = NULL;
    DsmApiSetCallback(handle, DS485_CONTAINER_REQUEST, ZONE_GROUP_ACTION_REQUEST, ZONE_GROUP_ACTION_REQUEST_ACTION_CALL_SCENE, &callback);

    DeviceStatisticsRequest_get(handle, dsid, 65, 0xff);

    sleep(10);

    ZoneCount(handle, broadcast, NULL);

    sleep(3600);
}
